<?php

/**
 * settings page
 */
class wtb_idf_calculator_settings
{
	const SETTINGS_KEY = 'wtb_idf_calculator';

	/**
	 * return currently installed plugin version
	 * @return mixed
	 */
	static function getInstalledVersion()
	{
		return get_option('wtb_idf_calculator_version', 0);
	}
	
	/**
	 * get plugin settings
	 * @return array
	 */
	static function getSettings()
	{
		return get_option(self::SETTINGS_KEY, array());
	}
	
	/**
	 * register settings for our plugin
	 */
	function registerSettings()
	{
		//register our settings
		register_setting('wtb_idf_calculator_settings_main_group', self::SETTINGS_KEY, array($this, 'sanitize'));
		
        // main section
		add_settings_section(
			'wtb_idf_calculator_settings_main_section',
			__('Main settings', 'wtb_idf_calculator'),
			null,
			'wtb_idf_calculator_settings'
		);	
        
		add_settings_field(
			'google_domain', 
			__('Google domain', 'wtb_idf_calculator'),
			array($this, 'create_an_google_domain_field'), 
			'wtb_idf_calculator_settings',
			'wtb_idf_calculator_settings_main_section'	
		);
        
		// custom post types
		add_settings_section(
			'wtb_idf_calculator_settings_cpt_section',
			__('Activate in custom post types', 'wtb_idf_calculator'),
			null,
			'wtb_idf_calculator_settings'
		);	

		add_settings_field(
			'cpt', 
			__('Custom post types', 'wtb_idf_calculator'),
			array($this, 'create_an_custom_post_types_fields'), 
			'wtb_idf_calculator_settings',
			'wtb_idf_calculator_settings_cpt_section'	
		);
	}
	
	/**
	 * renders plugin settings page
	 */
	function renderSettingsPage()
	{
		if ( !current_user_can( 'manage_options' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		} ?>

		<div class="wrap">
			<?php screen_icon(); ?>
			<h2><?php echo __('WDF*IDF Top10 Calculator settings', 'wtb_idf_calculator'); ?></h2>
			
			<form method="post" action="options.php">
				
				<?php
					settings_fields('wtb_idf_calculator_settings_main_group');	
					do_settings_sections('wtb_idf_calculator_settings');
				?>
				
				<?php submit_button(); ?>
			</form>
		</div>
		<?php 
	}
	
	/**
	 * check input from settings page
	 * @param type $input
	 * @return type
	 */
	public function sanitize($input)
	{
		return $input;
	}
	
	/**
	 * render custom post types selection fields
	 */
	public function create_an_google_domain_field()
	{
		$settings = self::getSettings();
		$cptSettings = !empty($settings['google_domain']) ? $settings['google_domain'] : '.com';
        ?>
            
            <select name="<?php echo self::SETTINGS_KEY ?>[google_domain]">
                <option value=".com" <?php echo selected($cptSettings, '.com') ?>>.com</option>
                <option value=".co.uk" <?php echo selected($cptSettings, '.co.uk') ?>>.co.uk</option>
                <option value=".co.id" <?php echo selected($cptSettings, '.co.id') ?>>.co.id</option>
                <option value=".co.in" <?php echo selected($cptSettings, '.co.in') ?>>.co.in</option>
                <option value=".co.jp" <?php echo selected($cptSettings, '.co.jp') ?>>.co.jp</option>
                <option value=".co.th" <?php echo selected($cptSettings, '.co.th') ?>>.co.th</option>
                <option value=".com.br" <?php echo selected($cptSettings, '.com.br') ?>>.com.br</option>
                <option value=".com.pk" <?php echo selected($cptSettings, '.com.pk') ?>>.com.pk</option>
                <option value=".com.bd" <?php echo selected($cptSettings, '.com.bd') ?>>.com.bd</option>
                <option value=".com.ng" <?php echo selected($cptSettings, '.com.ng') ?>>.com.ng</option>
                <option value=".com.mx" <?php echo selected($cptSettings, '.com.mx') ?>>.com.mx</option>
                <option value=".com.ph" <?php echo selected($cptSettings, '.com.ph') ?>>.com.ph</option>
                <option value=".com.fn" <?php echo selected($cptSettings, '.com.fn') ?>>.com.fn</option>
                <option value=".com.tr" <?php echo selected($cptSettings, '.com.tr') ?>>.com.tr</option>
                <option value=".com.ua" <?php echo selected($cptSettings, '.com.ua') ?>>.com.ua</option>
                <option value=".ca" <?php echo selected($cptSettings, '.ca') ?>>.ca</option>
                <option value=".es" <?php echo selected($cptSettings, '.es') ?>>.es</option>
                <option value=".de" <?php echo selected($cptSettings, '.de') ?>>.de</option>
                <option value=".at" <?php echo selected($cptSettings, '.at') ?>>.at</option>
                <option value=".fr" <?php echo selected($cptSettings, '.fr') ?>>.fr</option>
                <option value=".it" <?php echo selected($cptSettings, '.it') ?>>.it</option>
                <option value=".nl" <?php echo selected($cptSettings, '.nl') ?>>.nl</option>
                <option value=".ru" <?php echo selected($cptSettings, '.ru') ?>>.ru</option>
                <option value=".lt" <?php echo selected($cptSettings, '.lt') ?>>.lt</option>
                <option value=".lv" <?php echo selected($cptSettings, '.lv') ?>>.lv</option>
                <option value=".ee" <?php echo selected($cptSettings, '.ee') ?>>.ee</option>
                <option value=".fi" <?php echo selected($cptSettings, '.fi') ?>>.fi</option>
                <option value=".pl" <?php echo selected($cptSettings, '.pl') ?>>.pl</option>
                <option value=".ro" <?php echo selected($cptSettings, '.ro') ?>>.ro</option>
                <option value=".ch" <?php echo selected($cptSettings, '.ch') ?>>.ch</option>
                <option value=".se" <?php echo selected($cptSettings, '.se') ?>>.se</option>
            </select>
            
        <?php
		
		echo '<br />';
		echo '<br />';
    }
    
    /**
     * render main settings fields
     */
    public function create_an_custom_post_types_fields()
    {
        $settings = self::getSettings();
		$cptSettings = !empty($settings['cpt']) ? (array)$settings['cpt'] : array();
		foreach (get_post_types() as $cpt) {
			if (!in_array($cpt, array('attachment', 'revision', 'nav_menu_item'))) { ?>
				<input type="hidden" name="<?php echo self::SETTINGS_KEY ?>[cpt][<?php echo $cpt ?>]" value="-1" />
				<label>
					<input type="checkbox" name="<?php echo self::SETTINGS_KEY ?>[cpt][<?php echo $cpt ?>]" 
							<?php checked(!empty($cptSettings[$cpt]) and $cptSettings[$cpt] == 1)  ?>
						   value="1" /> 
					<?php echo $cpt; ?>
				</label><br />
				<?php
			}
		}
		
		echo '<br />';
		echo '<br />';
    }
	
}